==> shovel-0.0.3.tgz: package/lib/console.js <==
var celery = require('celeri'),
service = require('./service'),
_ = require('underscore'),
config = require('yaconfig').file('./shovel.json');


//TODO: oh god this shit's ugly.

function unableToFetch()
{
    return console.log('Unable to fetch'.red);
}


function simpleLoad(uri, label, ignoreSuccess)
{
    var ops = {};

    if(typeof uri == 'string')
    {
        ops.path = uri;
    }
    else
    {
        ops = uri;
    }

    var loading = celery.loading(label ? label + ': ' : null);
    
    service.request(ops, function(err, result) 
    { 
        loading.done(null, !ignoreSuccess ? result && !result.errors : undefined);
    });
}


function _apps(appName)
{
    var channel = '/my/apps/';

    if(appName) channel += appName + '/';

    return channel;
}


celery.on({

     /**
      * list apps 
      */
     
     'list': function()
     {
        var loading = celery.loading();

        
        service.request(_apps(), function(err, response)
        {
            loading.done();
            
            if(err || response.errors) return unableToFetch();
            
            var table = [];
            
            celery.drawTable(response.result, {
                columns: {
                    name: 20,
                    /*hostname: {
                        get: function(app)
                        {
                            if(!app.nodes.length) return "".yellow;
                            
                            return _.map(app.nodes, function(node)
                            {
                                //TODO: node name (hostname)
                                //return node.statename.green;
                                return node.hostname.green;
                            }).join('\n');
                        }
                    },*/
                    /*uptime: {
                        get: function(app)
                        {
                            if(!app.nodes.length) return "".yellow;
                            
                            return _.map(app.nodes, function(node)
                            {
                                //TODO: node name (hostname)
                                return node.description.green;
                            }).join('\n');
                        }
                    },*/
                    hostname: {
                        get: function(app)
                        {
                            if(!app.nodes.length) return "stopped".yellow;
                            
                            return _.map(app.nodes, function(node)
                            {
                                //TODO: node name (hostname)
                                return (node.hostname || '').green;
                            }).join('\n');
                        },
                        align:'left'
                    }
                    
                }
            });
        });
     },
    
    
    
    /**
     */
     
    ':appName history': function(data)
    {
        
        service.request(_apps(data.appName) + '/checkout/history', function(err, response)
        {
            if(err || response.errors) return unableToFetch();
            
            if(!response.result.length) return console.log('No commits');
            
            
            celery.drawTable(response.result, {
                columns: {
                    label: {
                        width: 20,
                        get: function(checkout)
                        {
                            return data.appName;// + '@' + checkout.package.version;
                        }
                    },
                    lastUsed: {
                        get: function(checkout)
                        {
                            return new Date(checkout.lastUsed);
                        }
                    }
                }
                
                
            });
        });
    },
    
    /**
     * node-inspector
     */
     
    'debug :appName': function()
    {
        //test
    },
    
    /**
     * good for v1 v2 apis
     */

    'clone :appName :newName': function()
    {
        console.log('cloning');
    },
    
    /**
     */
     
    ':appName rollback': function(data)
    {
        
        service.request(_apps(data.appName) + '/rollback', function(err, response)
        {
            if(err || response.errors) return unableToFetch();
            
            console.log('Rolling back to previous version');
        });
    },
    
    /**
     * returns the app logs
     */
     
    ':appName logs': function(data)
    {
        console.log("LOGS");
    },
    
    /**
     */
     
    ':appName processes': function(data)
    {
    },
    
    /**
     */
     
    ':appName checkout :repository': function(data)
    {
        celery.emit(data.appName+' checkout', { repository: data.repository });
    },
    
    /**
     */
     
    ':appName checkout': function(data)
    {   
        if(data.repository) data.repository = decodeURIComponent(data.repository);

        simpleLoad({ path: _apps(data.appName) + '/checkout', method: 'post', data: { repository: data.repository }}, 'Checking out '+data.appName);
    },
    
    /**
     * Starts interactive console with app - stdin
     */
     
    ':appName console': function(data)
    {
        console.log("TODO");
    },
    
    /**
     */
     
    ':appName logs tail': function(data)
    {
        console.log("TAIL LOGS");
    },
    
    /**
     * starts an app
     */
     
    'start :appName': function(data)
    {
        simpleLoad(_apps(data.appName) + '/start', 'Starting ' + data.appName, true);
    },

    /**
     * sets application cli args
     */

    'set :appName args :args': function(data)
    {
        console.log(data.args);
    },
    
    /**
     * stops an app
     */
     
    'stop :appName': function(data)
    {
        simpleLoad(_apps(data.appName) + '/stop', 'Stopping ' + data.appName, true);
    },
    
    /** 
     * restart target app
     */
     
    'restart :appName': function(data)
    {
        simpleLoad(_apps(data.appName) + '/restart', 'Restarting ' + data.appName, true);
    },
    
    
    
    /**
     * creates a new app
     */
     
    'create :appName': function(data)
    {
        service.request({ path: _apps(), method: 'post', data: { name: data.appName } }, function(err, result)
        {
            if(result.errors) return console.log('Unable to create app'.red);
            
            console.log('Successfuly created app');
        });
    },
    
    /**
     */
     
    'remove :appName': function(data)
    {
        service.request({ path: _apps(data.appName) + '/remove' }, function(err, result)
        {
            if(result.errors) return console.log('Unable to delete'.red);
            
            console.log('Successfuly deleted app');
        })
    },
    
    
    /**
     * restart all
     */
     
    'restart': function()
    {
        console.log("TODO");
    },
    /**
     */
     
    'help': function()
    {
        //celery.openHelp(__dirname + '/help.txt');
    },
    
    /**
     * shovel version
     */
     
    'version': function()
    {
    },
    
    /**
     */
     
    'signup': function()
    {
        celery.auth(function(user, pass)
        {
            console.log(user);
            console.log(pass);
        });
    },
    
    /**
     */
     
    'logout': function()
    {
        console.log("logged out".grey);

        config.set('auth', null);
        auth();
    }
});


function signup(email, user, pass)
{

    service.signup(email, user, pass, function(err, result)
    {
        if(err || result.errors)
        {
            console.error('Unable to signup'.red);
        }
        else
        {
            loggedIn();
        }
    });
}

function loggedIn()
{
    console.ok('logged in as %s'.green, service.username);
}

function auth()
{
    if(config.get('auth'))
    {
        service.setAuth(config.get('auth:username'), config.get('auth:password'));

        return loggedIn();
    }

    celery.auth(function(user, pass)
    {
        service.setAuth(user, pass);
        
        
        service.validate(function(err, result)
        {
            if(err) return console.log('Unable to validate credentials'.red); 
            
            if(result.errors)
            {
                celery.confirm('Auth credits are incorrect. Would you like to signup?', function(yes)
                {
                    if(yes)
                    {
                        return celery.prompt('Email: ', function(email)
                        {
                            signup(email, user, pass);
                        });
                    }

                    auth();
                });
            }
            else
            {
                config.set('auth', { username: user, password: pass })
                loggedIn();
            }
        });
    });    

    
}


auth();




celery.open();

celery.parse(process.argv);

==> shovel-0.0.5.tgz: package/lib/console.js <==
var celery = require('celeri'),
service = require('./service'),
_ = require('underscore'),
config = require('yaconfig').file('/etc/shovel.json');


//TODO: oh god this shit's ugly.

function unableToFetch()
{
    return console.log('Unable to fetch'.red);
}


function simpleLoad(uri, label, ignoreSuccess)
{
    var ops = {};

    if(typeof uri == 'string')
    {
        ops.path = uri;
    }
    else
    {
        ops = uri;
    }

    var loading = celery.loading(label ? label + ': ' : null);
    
    service.request(ops, function(err, result) 
    { 
        loading.done(null, !ignoreSuccess ? result && !result.errors : undefined);
    });
}


function _apps(appName)
{
    var channel = '/my/apps/';

    if(appName) channel += appName + '/';

    return channel;
}


celery.on({

     /**
      * list apps 
      */
     
     'list': function()
     {
        var loading = celery.loading();

        
        service.request(_apps(), function(err, response)
        {
            loading.done();
            
            if(err || response.errors) return unableToFetch();
            
            var table = [];
            
            celery.drawTable(response.result, {
                columns: {
                    name: 20,
                    /*hostname: {
                        get: function(app)
                        {
                            if(!app.nodes.length) return "".yellow;
                            
                            return _.map(app.nodes, function(node)
                            {
                                //TODO: node name (hostname)
                                //return node.statename.green;
                                return node.hostname.green;
                            }).join('\n');
                        }
                    },*/
                    /*uptime: {
                        get: function(app)
                        {
                            if(!app.nodes.length) return "".yellow;
                            
                            return _.map(app.nodes, function(node)
                            {
                                //TODO: node name (hostname)
                                return node.description.green;
                            }).join('\n');
                        }
                    },*/
                    hostname: {
                        get: function(app)
                        {
                            if(!app.nodes || !app.nodes.length) return "stopped".yellow;
                            
                            return _.map(app.nodes, function(node)
                            {
                                //TODO: node name (hostname)
                                return (node.hostname || '').green;
                            }).join('\n');
                        },
                        align:'left'
                    }
                    
                }
            });
        });
     },
    
    
    
    /**
     */
     
    ':appName history': function(data)
    {
        
        service.request(_apps(data.appName) + '/checkout/history', function(err, response)
        {
            if(err || response.errors) return unableToFetch();
            
            if(!response.result.length) return console.log('No commits');
            
            
            celery.drawTable(response.result, {
                columns: {
                    label: {
                        width: 20,
                        get: function(checkout)
                        {
                            return data.appName;// + '@' + checkout.package.version;
                        }
                    },
                    lastUsed: {
                        get: function(checkout)
                        {
                            return new Date(checkout.lastUsed);
                        }
                    }
                }
                
                
            });
        });
    },
    
    /**
     * node-inspector
     */
     
    'debug :appName': function()
    {
        //test
    },
    
    /**
     * good for v1 v2 apis
     */

    'clone :appName :newName': function()
    {
        console.log('cloning');
    },
    
    /**
     */
     
    ':appName rollback': function(data)
    {
        
        service.request(_apps(data.appName) + '/rollback', function(err, response)
        {
            if(err || response.errors) return unableToFetch();
            
            console.log('Rolling back to previous version');
        });
    },
    
    /**
     * returns the app logs
     */
     
    ':appName logs': function(data)
    {
        console.log("LOGS");
    },
    
    /**
     */
     
    ':appName processes': function(data)
    {
    },
    
    /**
     */
     
    ':appName checkout :repository': function(data)
    {
        celery.emit(data.appName+' checkout', { repository: data.repository });
    },
    
    /**
     */
     
    ':appName checkout': function(data)
    {   
        if(data.repository) data.repository = decodeURIComponent(data.repository);

        simpleLoad({ path: _apps(data.appName) + '/checkout', method: 'post', data: { repository: data.repository }}, 'Checking out '+data.appName);
    },
    
    /**
     * Starts interactive console with app - stdin
     */
     
    ':appName console': function(data)
    {
        console.log("TODO");
    },
    
    /**
     */
     
    ':appName logs tail': function(data)
    {
        console.log("TAIL LOGS");
    },
    
    /**
     * starts an app
     */
     
    'start :appName': function(data)
    {
        simpleLoad(_apps(data.appName) + '/start', 'Starting ' + data.appName, true);
    },

    /**
     * sets application cli args
     */

    'set :appName args :args': function(data)
    {
        console.log(data.args);
    },
    
    /**
     * stops an app
     */
     
    'stop :appName': function(data)
    {
        simpleLoad(_apps(data.appName) + '/stop', 'Stopping ' + data.appName, true);
    },
    
    /** 
     * restart target app
     */
     
    'restart :appName': function(data)
    {
        simpleLoad(_apps(data.appName) + '/restart', 'Restarting ' + data.appName, true);
    },
    
    
    
    /**
     * creates a new app
     */
     
    'create :appName': function(data)
    {
        service.request({ path: _apps(), method: 'post', data: { name: data.appName } }, function(err, result)
        {
            if(result.errors) return console.log('Unable to create app'.red);
            
            console.log('Successfuly created app');
        });
    },
    
    /**
     */
     
    'remove :appName': function(data)
    {
        service.request({ path: _apps(data.appName) + '/remove' }, function(err, result)
        {
            if(result.errors) return console.log('Unable to delete'.red);
            
            console.log('Successfuly deleted app');
        })
    },
    
    
    /**
     * restart all
     */
     
    'restart': function()
    {
        console.log("TODO");
    },
    /**
     */
     
    'help': function()
    {
        //celery.openHelp(__dirname + '/help.txt');
    },
    
    /**
     * shovel version
     */
     
    'version': function()
    {
    },
    
    /**
     */
     
    'signup': function()
    {
        celery.auth(function(user, pass)
        {
            console.log(user);
            console.log(pass);
        });
    },
    
    /**
     */
     
    'logout': function()
    {
        console.log("logged out".grey);

        config.set('auth', null);
        auth();
    }
});


function signup(email, user, pass)
{

    service.signup(email, user, pass, function(err, result)
    {
        if(err || result.errors)
        {
            console.error('Unable to signup'.red);
        }
        else
        {
            loggedIn();
        }
    });
}

function loggedIn()
{
    console.ok('logged in as %s'.green, service.username);
}

function auth()
{
    if(config.get('auth'))
    {
        service.setAuth(config.get('auth:username'), config.get('auth:password'));

        return loggedIn();
    }

    celery.auth(function(user, pass)
    {
        service.setAuth(user, pass);
        
        
        service.validate(function(err, result)
        {
            if(err) return console.log('Unable to validate credentials'.red); 
            
            if(result.errors)
            {
                celery.confirm('Auth credits are incorrect. Would you like to signup?', function(yes)
                {
                    if(yes)
                    {
                        return celery.prompt('Email: ', function(email)
                        {
                            signup(email, user, pass);
                        });
                    }

                    auth();
                });
            }
            else
            {
                config.set('auth', { username: user, password: pass })
                loggedIn();
            }
        });
    });    

    
}


auth();




celery.open();

celery.parse(process.argv);
